/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2019 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXAUDIO, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef AudioAdapter_H
#define AudioAdapter_H

#include <stdunk.h>
#include <portcls.h>
#include <ksdebug.h>

#include "Debug.h"

typedef struct _PHYSICALCONNECTIONTABLE
{
  ULONG topologyIn_;
  ULONG topologyOut_;
  ULONG waveIn_;
  ULONG waveOut_;
} PHYSICALCONNECTIONTABLE, *PPHYSICALCONNECTIONTABLE;


DEFINE_GUID(IID_AudioAdapter, 0x49afe056, 0x33b4, 0x4017, 0xab, 0x61, 0x83, 0x22, 0x58, 0x3f, 0x10, 0x22);


#define MAX_TOPOLOGY_NODES 20


class AudioAdapter :
  public IAdapterPowerManagement,
  public CUnknown    
{
  private:
 
  PPORTWAVECYCLIC portWaveCyclic_;
  
  PSERVICEGROUP serviceGroup_;

  DEVICE_OBJECT *device_;      
  
  DEVICE_POWER_STATE powerState_;


  protected:

  BOOL mute_[MAX_TOPOLOGY_NODES];

  LONG volume_[MAX_TOPOLOGY_NODES];
  
  ULONG mux_;
  public:

  //
  // Default CUnknown
  //
  
  DECLARE_STD_UNKNOWN();

  DEFINE_STD_CONSTRUCTOR(AudioAdapter);
  
  ~AudioAdapter();

  //
  // Default IAdapterPowerManagement
  //
  
  IMP_IAdapterPowerManagement;

  NTSTATUS init(PDEVICE_OBJECT DeviceObject);

  PDEVICE_OBJECT getDeviceObject();

  PUNKNOWN *getWavePortCyclic();

  void mixerReset();
  
  void setWaveServiceGroup(PSERVICEGROUP serviceGroup);

  BOOL getMute(ULONG node);

  void setMute(ULONG node, BOOL value);

  ULONG getMux();

  void setMux(ULONG node);
        
  LONG getVolume(ULONG node);

  void setVolume(ULONG node, LONG Value);
};

NTSTATUS CreateAudioAdapter(PUNKNOWN *AudioAdapter, REFCLSID,
                                 PUNKNOWN outer, POOL_TYPE poolType);

#endif
