/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2019 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXAUDIO, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "MiniPort.h"

//
// Connections path inside Wave Topology
//
// NULL -> WAVE_ADC -> NULL
// NULL -> WAVE_DAC -> NULL
//

PCCONNECTION_DESCRIPTOR MiniPort::connections_[] =
{
  //
  // source NODE    source PIN                 dest. NODE       dest. PIN
  //
  
  {PCFILTER_NODE,   KSPIN_WAVE_CAPTURE_SOURCE, KSNODE_WAVE_ADC, 1},
  {KSNODE_WAVE_ADC, 0,                         PCFILTER_NODE,   KSPIN_WAVE_CAPTURE_SINK},

  {PCFILTER_NODE,   KSPIN_WAVE_RENDER_SINK,    KSNODE_WAVE_DAC, 1},
  {KSNODE_WAVE_DAC, 0,                         PCFILTER_NODE,   KSPIN_WAVE_RENDER_SOURCE},
};


KSDATARANGE_AUDIO MiniPort::pinDataRangesStream_[] =
{
  {
    {
      sizeof(KSDATARANGE_AUDIO),
      0,
      0,
      0,
      STATICGUIDOF(KSDATAFORMAT_TYPE_AUDIO),
      STATICGUIDOF(KSDATAFORMAT_SUBTYPE_PCM),
      STATICGUIDOF(KSDATAFORMAT_SPECIFIER_WAVEFORMATEX)
    },

    INTERNAL_CHANNELS,
    INTERNAL_RESOLUTION,
    INTERNAL_RESOLUTION,
    INTERNAL_SAMPLE_RATE,
    INTERNAL_SAMPLE_RATE
  },
};

PKSDATARANGE MiniPort::pinDataRangePointersStream_[] =
{
  PKSDATARANGE(&pinDataRangesStream_[0])
};


KSDATARANGE MiniPort::pinDataRangesBridge_[] =
{
  {
    sizeof(KSDATARANGE),
    0,
    0,
    0,
    STATICGUIDOF(KSDATAFORMAT_TYPE_AUDIO),
    STATICGUIDOF(KSDATAFORMAT_SUBTYPE_ANALOG),
    STATICGUIDOF(KSDATAFORMAT_SPECIFIER_NONE)
  }
};

PKSDATARANGE MiniPort::pinDataRangePointersBridge_[] =
{
  &pinDataRangesBridge_[0]
};


//
// Nodes hidden inside Wave Topology (MiniPort)
// 

PCNODE_DESCRIPTOR MiniPort::nodes_[] =
{
  //
  // KSNODE_WAVE_ADC
  //
  
  {
    0,                      // Flags
    NULL,                   // AutomationTable
    &KSNODETYPE_ADC,        // Type
    NULL                    // Name
  },
 
  //
  // KSNODE_WAVE_DAC
  //

  {
    0,                      // Flags
    NULL,                   // AutomationTable
    &KSNODETYPE_DAC,        // Type
    NULL                    // Name
  }
};


//
// Pins exposed by Wave Topology (MiniPort)
//

PCPIN_DESCRIPTOR MiniPort::pins_[] =
{
  //
  // KSPIN_WAVE_CAPTURE_SINK
  //
    
  {
    MAX_OUTPUT_STREAMS,
    MAX_OUTPUT_STREAMS,
    0,
    NULL,
    {
      0,
      NULL,
      0,
      NULL,

      SIZEOF_ARRAY(pinDataRangePointersStream_),
      pinDataRangePointersStream_,
      
      KSPIN_DATAFLOW_OUT,
      KSPIN_COMMUNICATION_SINK,
      &KSCATEGORY_AUDIO,
      &KSAUDFNAME_RECORDING_CONTROL,  
      0
    }
  },
    
  //
  // KSPIN_WAVE_CAPTURE_SOURCE
  //
  
  {
    0,
    0,
    0,
    NULL,
    {
      0,
      NULL,
      0,
      NULL,

      SIZEOF_ARRAY(pinDataRangePointersBridge_),
      pinDataRangePointersBridge_,
      
      KSPIN_DATAFLOW_IN,
      KSPIN_COMMUNICATION_NONE,
      &KSCATEGORY_AUDIO,
      NULL,
      0
    }
  },
  
  //
  // KSPIN_WAVE_RENDER_SINK
  //

  {
    MAX_INPUT_STREAMS,
    MAX_INPUT_STREAMS, 
    0,
    NULL,
    {
      0,
      NULL,
      0,
      NULL,

      SIZEOF_ARRAY(pinDataRangePointersStream_),
      pinDataRangePointersStream_,
      
      KSPIN_DATAFLOW_IN,
      KSPIN_COMMUNICATION_SINK,
      &KSCATEGORY_AUDIO,
      NULL,
      0
    }
  },
  
  //
  // KSPIN_WAVE_RENDER_SOURCE
  //

  {
    0,
    0,
    0,
    NULL,
    {
      0,
      NULL,
      0,
      NULL,

      SIZEOF_ARRAY(pinDataRangePointersBridge_),
      pinDataRangePointersBridge_,
      
      KSPIN_DATAFLOW_OUT,
      KSPIN_COMMUNICATION_NONE,
      &KSCATEGORY_AUDIO,
      NULL,
      0
    }
  }
};


PCFILTER_DESCRIPTOR MiniPort::filterDescriptor_ =
{
  0,                                  // Version
  &automationWaveFilter_,             // AutomationTable
  
  sizeof(PCPIN_DESCRIPTOR),           // PinSize
  SIZEOF_ARRAY(pins_),                // PinCount
  pins_,                              // Pins
  
  sizeof(PCNODE_DESCRIPTOR),          // NodeSize
  SIZEOF_ARRAY(nodes_),               // NodeCount
  nodes_,                             // Nodes
  
  SIZEOF_ARRAY(connections_),         // ConnectionCount
  connections_,                       // Connections

  0,                                  // CategoryCount
  NULL                                // Categories  - use defaults (audio, render, capture)
};


//
// Automation table for properties.
//

PCPROPERTY_ITEM MiniPort::propertiesWaveFilter_[] =
{
  {
    &KSPROPSETID_General,
    KSPROPERTY_GENERAL_COMPONENTID,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  },
  {
    &KSPROPSETID_Pin,
    KSPROPERTY_PIN_PROPOSEDATAFORMAT,
    KSPROPERTY_TYPE_SET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  },
  
  //
  // Custom NX property set. We can send requests for
  // for them via DeviceIoControl().
  //
  
  {
    &KSPROPSETID_nxaudio,
    KSPROPERTY_NXAUDIO_STATUS,
    KSPROPERTY_TYPE_GET,
    propertyHandler
  },
  
};

DEFINE_PCAUTOMATION_TABLE_PROP(MiniPort::automationWaveFilter_, propertiesWaveFilter_);
