/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2019 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXAUDIO, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Stream_H
#define Stream_H

#include <stdunk.h>
#include <portcls.h>
#include <ksdebug.h>
#include <limits.h>

#include "AudioAdapter.h"
#include "MiniPort.h"

class MiniPort;

class Stream : 
  public IMiniportWaveCyclicStream,
  public IDmaChannel,
  public CUnknown
{
  protected:

  MiniPort *miniPort_;
  
  //
  // Actual stream format.
  //
  
  USHORT blockAlign_;  

  //
  // Stream state.
  //
  
  BOOLEAN capture_;
  
  KSSTATE ksState_;
  
  //
  // Fake DMA buffer.
  //
  
  ULONG dmaPos_;
  
  void *dmaBuffer_;
  
  ULONG dmaBufferSize_;
  
  ULONG dmaMovementRate_;
  
  ULONGLONG dmaTimeStamp_;
  
  //
  // Fake DMA time synchronization.
  //
  
  PRKDPC dpc_;
  
  PKTIMER timer_;

  ULONGLONG timeCarryForward_;

  ULONG byteCarryForward_;

public:

  DECLARE_STD_UNKNOWN();

  Stream(PUNKNOWN outerUnknown);

  ~Stream();

  IMP_IMiniportWaveCyclicStream;
  
  IMP_IDmaChannel;  
  
  NTSTATUS init(MiniPort *miniPort, ULONG channel, 
                    BOOLEAN capture, PKSDATAFORMAT dataFormat);
                    
  void cleanBuffer();

  //
  // Functions incrementing a counter of number of wraparounds of pointers, without overlfows.
  // They should be in MiniPort class, but this adds latency when using and might creat glitches in audio.
  //

  void changeLapWrite();

  void changeLapRead();

  friend class MiniPort;
};

#endif