/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2019 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXAUDIO, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

//
// This tool is for testing purpose only. It tries to reproduce
// crash by creating many processes, where use sound card at the
// same time.
//

#include <cstdio>
#include <cstdlib>
#include <windows.h>
#include <mmsystem.h>
#include "dsound.h"

const GUID GUID_NULL = { 0, 0, 0, { 0, 0, 0, 0, 0, 0, 0, 0 } }; 

#define FAIL(X) if (X) goto fail
#define NTFAIL(X) if ((ret = (X))) goto fail

int main(int argc, char **argv)
{
  IDirectSound *ds;
  IDirectSoundBuffer *buf;
  
  DWORD ret = 0;

  DWORD bufferSize = 1024 * 1024;

  BYTE buffer[1024];
  
  BYTE *bufferPtr = (BYTE *) &buffer;

  #define NUM_TESTS        16
  #define TESTS_PER_THREAD 16
  #define MAX_THREADS      8
  
  for (int i = 0; i < 1024; i++)
  {
    buffer[i] = rand();
  }
  
  //
  // Master process creates slaves.
  //
  
  if (argc == 1)
  {
    for (int testNo = 0; testNo < NUM_TESTS; testNo ++)
    {
      HANDLE handles[MAX_THREADS] = {0};
    
      for (int i = 0; i < MAX_THREADS; i++)
      {
        STARTUPINFO si = {sizeof(STARTUPINFO)};
    
        PROCESS_INFORMATION pi = {0};

        FAIL(CreateProcess(0, (char *) "dstest --slave", 
                               0, 0, TRUE, 0, 0, 0, &si, &pi) == 0);
                           
        handles[i] = pi.hProcess;
      
        CloseHandle(pi.hThread);
      }
    
      WaitForMultipleObjects(MAX_THREADS, handles, TRUE, INFINITE);
        
      for (int i = 0; i < MAX_THREADS; i++)
      {
        CloseHandle(handles[i]);
      }
    }  
  }    
                           
  //
  // Slave process do real work.
  //
  
  else
  {  
    for (int i = 0; i < TESTS_PER_THREAD; i++)
    {
      DWORD pid = GetCurrectProcessId();
      
      //
      // Create direct sound object.
      //

      printf("[%d] -> DirectSoundCreate()...\n", pid);
      
      NTFAIL(DirectSoundCreate(0, &ds, 0));
      
      printf("[%d] <- DirectSoundCreate()...\n", pid);

      NTFAIL(ds -> SetCooperativeLevel(GetDesktopWindow(), DSSCL_PRIORITY));
      
      //
      // Create buffer.
      //

      DSBUFFERDESC bufDesc;
      WAVEFORMATEX format;

      //
      // Set format.
      //

      format.wFormatTag      = WAVE_FORMAT_PCM;
      format.nSamplesPerSec  = 44100;
      format.wBitsPerSample  = 16;
      format.nChannels       = 2;
      format.nBlockAlign     = (format.wBitsPerSample / 8) * format.nChannels;
      format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign;
      format.cbSize          = 0;

      //
      // Create buffer.
      //

      bufDesc.dwSize          = sizeof(DSBUFFERDESC);
      bufDesc.dwFlags         = DSBCAPS_CTRLVOLUME;
      bufDesc.dwBufferBytes   = 1024 * 1024;
      bufDesc.dwReserved      = 0;
      bufDesc.lpwfxFormat     = &format;
      bufDesc.guid3DAlgorithm = GUID_NULL;
      
      printf("[%d] -> CreateSoundBuffer()...\n", pid);

      NTFAIL(ds -> CreateSoundBuffer(&bufDesc, &buf, 0));
        
      printf("[%d] <- CreateSoundBuffer()...\n", pid);

      NTFAIL(buf -> SetCurrentPosition(0));
      NTFAIL(buf -> SetVolume(DSBVOLUME_MAX));

      NTFAIL(buf -> Lock(0, 1024 * 1024, (void **) &bufferPtr,
                             (DWORD *) &bufferSize, 0, 0, 0));

      for (int i = 0; i < 1024 * 1024; i++)
      {
        bufferPtr[i] = rand();
      }
        
      NTFAIL(buf -> Unlock((void *) bufferPtr, bufferSize, 0, 0));

      //
      // Play sample.
      //
      
      printf("[%d] -> Play()...\n", pid);
      
      NTFAIL(buf -> Play(0, 0, 0));
        
      printf("[%d] <- Play()...\n", pid);

      //
      // Release direct sound object.
      //
      
      printf("[%d] -> Release()...\n", pid);
      
      buf -> Release();
                
      ds -> Release();
        
      printf("[%d] <- Release()...\n", pid);
    }
  }
  
  return 0;
  
  fail:
  
  fprintf(stderr, "HRESULT = 0x%x, LastError = %u\n", ret, GetLastError());
  
  return -1;
}
