/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2019 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXAUDIO, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#ifndef Topology_H       
#define Topology_H       

#include <stdunk.h>
#include <portcls.h>
#include <ksdebug.h>

#include "AudioAdapter.h"
#include "MiniPort.h"


#define KSPROPERTY_TYPE_ALL KSPROPERTY_TYPE_BASICSUPPORT | \
                            KSPROPERTY_TYPE_GET | \
                            KSPROPERTY_TYPE_SET

NTSTATUS CreateMiniPortTopology(PUNKNOWN *unknown, REFCLSID,
                                    PUNKNOWN outer, POOL_TYPE poolType);


class AudioAdapter;

class Topology : 
  public IMiniportTopology,
  public CUnknown
{
  protected:
  
  AudioAdapter *audioAdapter_;
  

  static KSDATARANGE pinDataRangesBridge_[];
  
  static PKSDATARANGE pinDataRangePointersBridge_[];
  
  static PCPIN_DESCRIPTOR pins_[];

  static PCNODE_DESCRIPTOR nodes_[];
  
  static PCPROPERTY_ITEM propertiesVolume_[];
  static PCPROPERTY_ITEM propertiesMute_[];
  static PCPROPERTY_ITEM propertiesMux_[];

  static PCCONNECTION_DESCRIPTOR connections_[];

  static PCPROPERTY_ITEM propertiesFilter_[];

  static PCFILTER_DESCRIPTOR filterDescriptor_;
  
  
  public:
  
  static const PHYSICALCONNECTIONTABLE physicalConnections_;
  
  static const PCAUTOMATION_TABLE automationVolume_;
  static const PCAUTOMATION_TABLE automationMute_;
  static const PCAUTOMATION_TABLE automationMux_;
  static const PCAUTOMATION_TABLE automationFilter_;
  

  DECLARE_STD_UNKNOWN();

  Topology(PUNKNOWN outer);

  ~Topology();

  IMP_IMiniportTopology;

  NTSTATUS Init(PUNKNOWN UnknownAdapter, PPORTTOPOLOGY port);

  //
  // Generic property handlers. These functionis call one of
  // handlers related to given request type.
  //
   
  static NTSTATUS propertyHandler(PPCPROPERTY_REQUEST request);
  
  static NTSTATUS propertyHandlerBasicSupport(PPCPROPERTY_REQUEST request, 
                                                  ULONG flags, DWORD propType);

  static NTSTATUS propertyHandlerFilter(PPCPROPERTY_REQUEST request);

  //
  // Property handlers for given request types.
  //
  
  NTSTATUS propertyHandlerMute(PPCPROPERTY_REQUEST request);

  NTSTATUS propertyHandlerMuxSource(PPCPROPERTY_REQUEST request);

  NTSTATUS propertyHandlerVolume(PPCPROPERTY_REQUEST request);
  
  NTSTATUS propertyHandlerCpuResource(PPCPROPERTY_REQUEST request);
};

#endif
