/**************************************************************************/
/*                                                                        */
/* Copyright (c) 2011, 2019 NoMachine, http://www.nomachine.com/.         */
/*                                                                        */
/* NXAUDIO, NX protocol compression and extensions to this software       */
/* are copyright of NoMachine. Redistribution and use of the present      */
/* software is allowed according to terms specified in the file LICENSE   */
/* which comes in the source distribution.                                */
/*                                                                        */
/* Check http://www.nomachine.com/licensing.html for applicability.       */
/*                                                                        */
/* NX and NoMachine are trademarks of Medialogic S.p.A.                   */
/*                                                                        */
/* All rights reserved.                                                   */
/*                                                                        */
/**************************************************************************/

#include "Topology.h"

const PHYSICALCONNECTIONTABLE Topology::physicalConnections_ =
{
  KSPIN_TOPO_WAVEOUT_SOURCE,  // TopologyIn
  KSPIN_TOPO_WAVEIN_DEST,     // TopologyOut
  KSPIN_WAVE_CAPTURE_SOURCE,  // WaveIn
  KSPIN_WAVE_RENDER_SOURCE    // WaveOut
};

//
// table with possibly formats range
//

KSDATARANGE Topology::pinDataRangesBridge_[] =
{
  {
    sizeof(KSDATARANGE),
    0,
    0,
    0,
    STATICGUIDOF(KSDATAFORMAT_TYPE_AUDIO),
    STATICGUIDOF(KSDATAFORMAT_SUBTYPE_ANALOG),
    STATICGUIDOF(KSDATAFORMAT_SPECIFIER_NONE)
  }
};

PKSDATARANGE Topology::pinDataRangePointersBridge_[] =
{
  &pinDataRangesBridge_[0]
};

//
// PINs description
//                      ------------
// -> WAVEOUT_SRC    -> | TOPOLOGY | -> LINE_OUT_DEST ->
// -> SYNTH_OUT_SRC  -> | NODE     |  
// -> MIC_SOURCE_SRC -> |          | -> WAVE_IN_DEST -> 
//                      ------------
//

PCPIN_DESCRIPTOR Topology::pins_[] =
{
  //
  // KSPIN_TOPO_WAVEOUT_SOURCE
  //

  {
    0,
    0,
    0,                                              // InstanceCount
    NULL,                                           // AutomationTable
    {                                               // KsPinDescriptor
      0,                                            // InterfacesCount
      NULL,                                         // Interfaces
      0,                                            // MediumsCount
      NULL,                                         // Mediums

      SIZEOF_ARRAY(pinDataRangePointersBridge_),    // DataRangesCount
      pinDataRangePointersBridge_,                  // DataRanges
      
      KSPIN_DATAFLOW_IN,                            // DataFlow
      KSPIN_COMMUNICATION_NONE,                     // Communication
      &KSCATEGORY_AUDIO,                            // Category
      NULL,                                         // Name
      0                                             // Reserved
    }
  },

  //
  // KSPIN_TOPO_SYNTHOUT_SOURCE
  //
  
  {
    0,
    0, 
    0,                                              // InstanceCount
    NULL,                                           // AutomationTable
    {                                               // KsPinDescriptor
      0,                                            // InterfacesCount
      NULL,                                         // Interfaces
      0,                                            // MediumsCount
      NULL,                                         // Mediums

      SIZEOF_ARRAY(pinDataRangePointersBridge_),    // DataRangesCount
      pinDataRangePointersBridge_,                  // DataRanges
      
      KSPIN_DATAFLOW_IN,                            // DataFlow
      KSPIN_COMMUNICATION_NONE,                     // Communication
      &KSNODETYPE_SYNTHESIZER,                      // Category
      &KSAUDFNAME_MIDI,                             // Name
      0                                             // Reserved
    }
  },

  //
  // KSPIN_TOPO_MIC_SOURCE
  //
  
  {
    0,
    0,
    0,                                              // InstanceCount
    NULL,                                           // AutomationTable
    {                                               // KsPinDescriptor
      0,                                            // InterfacesCount
      NULL,                                         // Interfaces
      0,                                            // MediumsCount
      NULL,                                         // Mediums

      SIZEOF_ARRAY(pinDataRangePointersBridge_),    // DataRangesCount
      pinDataRangePointersBridge_,                  // DataRanges
      
      KSPIN_DATAFLOW_IN,                            // DataFlow
      KSPIN_COMMUNICATION_NONE,                     // Communication
      &KSNODETYPE_MICROPHONE,                       // Category
      NULL,                                         // Name
      0                                             // Reserved
    }
  },

  //
  // KSPIN_TOPO_LINEOUT_DEST
  //
  
  {
    0,
    0,
    0,                                              // InstanceCount
    NULL,                                           // AutomationTable
    {                                               // KsPinDescriptor
      0,                                            // InterfacesCount
      NULL,                                         // Interfaces
      0,                                            // MediumsCount
      NULL,                                         // Mediums

      SIZEOF_ARRAY(pinDataRangePointersBridge_),    // DataRangesCount
      pinDataRangePointersBridge_,                  // DataRanges
      
      KSPIN_DATAFLOW_OUT,                           // DataFlow
      KSPIN_COMMUNICATION_NONE,                     // Communication
      &KSNODETYPE_SPEAKER,                          // Category
      NULL,                                         // Name
      0                                             // Reserved
    }
  },

  //
  // KSPIN_TOPO_WAVEIN_DEST
  //
  
  {
    0,
    0,
    0,                                              // InstanceCount
    NULL,                                           // AutomationTable
    {                                               // KsPinDescriptor
      0,                                            // InterfacesCount
      NULL,                                         // Interfaces
      0,                                            // MediumsCount
      NULL,                                         // Mediums

      SIZEOF_ARRAY(pinDataRangePointersBridge_),    // DataRangesCount
      pinDataRangePointersBridge_,                  // DataRanges
      
      KSPIN_DATAFLOW_OUT,                           // DataFlow
      KSPIN_COMMUNICATION_NONE,                     // Communication
      &KSCATEGORY_AUDIO,                            // Category
      NULL,                                         // Name
      0                                             // Reserved
    }
  }
};

//
// Automation table for volume property
//

PCPROPERTY_ITEM Topology::propertiesVolume_[] =
{
    {
    &KSPROPSETID_Audio,
    KSPROPERTY_AUDIO_VOLUMELEVEL,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
    },
    {
    &KSPROPSETID_Audio,
    KSPROPERTY_AUDIO_CPU_RESOURCES,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  }
};

DEFINE_PCAUTOMATION_TABLE_PROP(Topology::automationVolume_, propertiesVolume_);

//
// Automation table for mute property
//

PCPROPERTY_ITEM Topology::propertiesMute_[] =
{
  {
    &KSPROPSETID_Audio,
    KSPROPERTY_AUDIO_MUTE,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  },
  {
    &KSPROPSETID_Audio,
    KSPROPERTY_AUDIO_CPU_RESOURCES,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  }
};

DEFINE_PCAUTOMATION_TABLE_PROP(Topology::automationMute_, propertiesMute_);

//
// Automation table for mux property
//

PCPROPERTY_ITEM Topology::propertiesMux_[] =
{
  {
    &KSPROPSETID_Audio,
    KSPROPERTY_AUDIO_MUX_SOURCE,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_SET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  },
  {
    &KSPROPSETID_Audio,
    KSPROPERTY_AUDIO_CPU_RESOURCES,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandler
  }
};

DEFINE_PCAUTOMATION_TABLE_PROP(Topology::automationMux_, propertiesMux_);

//
// Topology nodes table
//

PCNODE_DESCRIPTOR Topology::nodes_[] =
{
  //
  // KSNODE_TOPO_WAVEOUT_VOLUME
  //
  
  {
    0,                      // Flags
    &automationVolume_,     // AutomationTable
    &KSNODETYPE_VOLUME,     // Type
    &KSAUDFNAME_WAVE_VOLUME // Name
  },

  //
  // KSNODE_TOPO_WAVEOUT_MUTE
  //
  
  {
    0,                      // Flags
    &automationMute_,       // AutomationTable
    &KSNODETYPE_MUTE,       // Type
    &KSAUDFNAME_WAVE_MUTE   // Name
  },

  //
  // KSNODE_TOPO_SYNTHOUT_VOLUME
  //
  
  {
    0,                      // Flags
    &automationVolume_,     // AutomationTable
    &KSNODETYPE_VOLUME,     // Type
    &KSAUDFNAME_MIDI_VOLUME // Name
  },

  //
  // KSNODE_TOPO_SYNTHOUT_MUTE
  //
  
  {
    0,                      // Flags
    &automationMute_,       // AutomationTable
    &KSNODETYPE_MUTE,       // Type
    &KSAUDFNAME_MIDI_MUTE   // Name
  },

  //
  // KSNODE_TOPO_MIC_VOLUME
  //
  
  {
    0,                      // Flags
    &automationVolume_,     // AutomationTable
    &KSNODETYPE_VOLUME,     // Type
    &KSAUDFNAME_MIC_VOLUME  // Name
  },

  //
  // KSNODE_TOPO_LINEOUT_MIX
  //
  
  {
    0,                      // Flags
    NULL,                   // AutomationTable
    &KSNODETYPE_SUM,        // Type
    NULL                    // Name
  },

  //
  // KSNODE_TOPO_LINEOUT_VOLUME
  //
  
  {
    0,                        // Flags
    &automationVolume_,       // AutomationTable
    &KSNODETYPE_VOLUME,       // Type
    &KSAUDFNAME_MASTER_VOLUME // Name
  },

  //
  // KSNODE_TOPO_WAVEIN_MUX
  //
  
  {
    0,                           // Flags
    &automationMux_,             // AutomationTable
    &KSNODETYPE_MUX,             // Type
    &KSAUDFNAME_RECORDING_SOURCE // Name
  }
};


//
// Connection paths beetwen nodes
//
// NULL nodes means, it is external node outside topology.
//
// NULL -> WAVEOUT_VOL  -> WAVEOUT_MUTE  \
// NULL -> SYNTHOUT_VOL -> SYNTHOUT_MUTE -> LINEOUT_MIX -> LINEOUT_VOL -> NULL
//
// NULL -> MIC_VOLUME   -> NULL
//
// On standard nodes 0 means OUTPUT and 1 means INPUT.
//

PCCONNECTION_DESCRIPTOR Topology::connections_[] =
{
  //
  //  FromNode,                 FromPin,                    ToNode,                      ToPin
  //
  
  {PCFILTER_NODE,               KSPIN_TOPO_WAVEOUT_SOURCE,  KSNODE_TOPO_WAVEOUT_VOLUME,  1},
  {KSNODE_TOPO_WAVEOUT_VOLUME,  0,                          KSNODE_TOPO_WAVEOUT_MUTE,    1},
  {KSNODE_TOPO_WAVEOUT_MUTE,    0,                          KSNODE_TOPO_LINEOUT_MIX,     1},
  
  {PCFILTER_NODE,               KSPIN_TOPO_SYNTHOUT_SOURCE, KSNODE_TOPO_SYNTHOUT_VOLUME, 1},
  {KSNODE_TOPO_SYNTHOUT_VOLUME, 0,                          KSNODE_TOPO_SYNTHOUT_MUTE,   1},
  {KSNODE_TOPO_SYNTHOUT_MUTE,   0,                          KSNODE_TOPO_LINEOUT_MIX,     2},
                                                                                          
  {KSNODE_TOPO_LINEOUT_MIX,     0,                          KSNODE_TOPO_LINEOUT_VOLUME,  1},
  {KSNODE_TOPO_LINEOUT_VOLUME,  0,                          PCFILTER_NODE,               KSPIN_TOPO_LINEOUT_DEST},

  {PCFILTER_NODE,               KSPIN_TOPO_MIC_SOURCE,      KSNODE_TOPO_MIC_VOLUME,      1},
  {KSNODE_TOPO_MIC_VOLUME,      0,                          PCFILTER_NODE,               KSPIN_TOPO_WAVEIN_DEST}
};

//
// Automation table for TopoFilter property (??)
//

PCPROPERTY_ITEM Topology::propertiesFilter_[] =
{
  {
    &KSPROPSETID_Jack,
    KSPROPERTY_JACK_DESCRIPTION,
    KSPROPERTY_TYPE_GET | KSPROPERTY_TYPE_BASICSUPPORT,
    propertyHandlerFilter
  }
};

DEFINE_PCAUTOMATION_TABLE_PROP(Topology::automationFilter_, propertiesFilter_);


//
// Topology filter descriptor.
//

PCFILTER_DESCRIPTOR Topology::filterDescriptor_ =
{
  0,                                  // Version
  &automationFilter_,                 // AutomationTable

  sizeof(PCPIN_DESCRIPTOR),           // PinSize
  SIZEOF_ARRAY(pins_),                // PinCount
  pins_,                              // Pins
  
  sizeof(PCNODE_DESCRIPTOR),          // NodeSize
  SIZEOF_ARRAY(nodes_),               // NodeCount
  nodes_,                             // Nodes
  
  SIZEOF_ARRAY(connections_),         // ConnectionCount
  connections_,                       // Connections
  
  0,                                  // CategoryCount
  NULL                                // Categories
};
